/** 
	A program in Qt
	that use OpenGL drawing primitives
	File: 4cubes.cpp

	Fraga 16/07/2002
**/
#include <qpainter.h>
#include "cubes.h"

Cube::Cube( QWidget *parent, const char *name )
        : QGLWidget( parent, name )
{
	signo = 1;	

	helix = body = 0;

	// setMouseTracking( TRUE );

	timer = new QTimer( this, "mytimer" );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	// setDoubleBuffer ( TRUE );  // This is enable by default
	// setRgba( TRUE );           // This is enable by default
	// setDepth( TRUE );          // This is enable by default
}


void Cube::timerDone( )
{
	static int i=0;

	helix += 10.0*signo;

	if ( helix >= 360.0 )  helix = 0.0;
	if ( helix <= -360.0 ) helix = 0.0;

	if ( i==2 ) {	
		body += 10.0*signo;

		if ( body >= 360.0 )  body = 0.0;
		if ( body <= -360.0 ) body = 0.0;
		i=0;
	}
	i++;
	updateGL( );
}

void Cube::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
    xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

//
// This method puts the initial coordinates each time window is resized
//
void Cube::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	glOrtho (-aspect, aspect, -1.0, 1.0, -1.0, 1.0);

	emit changeSize ( );
}

QSizePolicy Cube::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

/** 
	Function that generate a cube with center in (cx,cy,cz)
	with size value for its side
**/
void Cube::getcube ( float cx, float cy, float cz, float size, float v[8][3] ) 
{
	v[0][0] = v[3][0] = v[4][0] = v[7][0] = cx;
	v[0][1] = v[1][1] = v[2][1] = v[3][1] = cy;
	v[0][2] = v[1][2] = v[4][2] = v[5][2] = cz;
	v[1][0] = v[2][0] = v[5][0] = v[6][0] = cx + size;
	v[4][1] = v[5][1] = v[6][1] = v[7][1] = cy + size;
	v[2][2] = v[3][2] = v[6][2] = v[7][2] = cz + size;
}

void Cube::show_cube ( float v[8][3] )
{
	static GLfloat n[6][3] = {
		{ 0.0, 0.0, 1.0},
		{ 1.0, 0.0, 0.0},
		{ 0.0, 0.0,-1.0},
		{-1.0, 0.0, 0.0},
		{ 0.0, 1.0, 0.0},
		{ 0.0,-1.0, 0.0} };

	static GLint faces[6][4] = {
		{7,6,2,3},
		{6,5,1,2},
		{5,1,0,4},
		{4,0,3,7},
		{4,5,6,7},
		{0,1,2,3} };

	static GLfloat colors[6][3] = {
		{ 1.0, 1.0, 1.0}, // white
		{ 1.0, 0.0, 0.0}, // red
		{ 0.0, 1.0, 0.0}, // green
		{ 0.0, 0.0, 1.0}, // blue
		{ 1.0, 1.0, 0.0}, // yellow
		{ 1.0, 0.0, 1.0}  // magenta 
	};

	int i;

	for (i = 5; i >= 0; i--) {
		glBegin(GL_QUADS);
		// glBegin(GL_LINE_LOOP);
		glColor3fv( &colors[i][0] );
		glNormal3fv(&n[i][0]);
		glVertex3fv(&v[faces[i][0]][0]);
		glVertex3fv(&v[faces[i][1]][0]);
		glVertex3fv(&v[faces[i][2]][0]);
		glVertex3fv(&v[faces[i][3]][0]);
		glEnd();
	}
}

void Cube::paintGL(void)
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();             /* clear the matrix */
	glRotatef (xrot, 1.0, 0.0, 0.0);
	glRotatef (yrot, 0.0, 1.0, 0.0);

	/* viewing transformation  */
	glPushMatrix( );
	glScalef ( 0.3, 0.3, 0.3);      /* modeling transformation */ 

	glRotatef ( body, 0.0, 1.0, 0.0);

	glTranslatef ( 0.0, 0.0, 0.5);

	glPushMatrix( );
		glRotatef ( helix, 0.0, 0.0, 1.0);
		show_cube( c1 );
		show_cube( c2 );
		show_cube( c3 );
	glPopMatrix( );

	show_cube( c4 );
	show_cube( c5 );

	// Draw x axis:
	glBegin(GL_LINES);
		glColor3f( 1.0, 0.0, 0.0 );
		glVertex3f(-0.5, 0.0, 0.0);
		glVertex3f( 3.5, 0.0, 0.0);
	glEnd();
	
	glPopMatrix( );

	swapBuffers();
}

void Cube::initializeGL( ) 
{
	glClearColor (0.8, 0.8, 0.8, 0.0);	// Background to white

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-1.0, 1.0,-1.0, 1.0,-1.0, 1.0);

	// Helix cubes
	getcube ( -0.5,-0.5, 0.0, 1.0, c1 );
	getcube ( -1.5,-0.5, 0.0, 1.0, c2 );
	getcube (  0.5,-0.5, 0.0, 1.0, c3 );
	// 
	getcube ( -0.5,-0.5,-1.0, 1.0, c4 );
	getcube ( -0.5,-0.5,-2.0, 1.0, c5 );

	glEnable (GL_DEPTH_TEST);
}

